@@ -1,5 +1,5 @@
 /*
- * $Header: /cvsroot/pgsql/contrib/pgstattuple/pgstattuple.c,v 1.12.4.3 2010/04/02 16:17:31 tgl Exp $
+ * $PostgreSQL: pgsql/contrib/pgstattuple/pgstattuple.c,v 1.17 2004/10/15 22:39:38 tgl Exp $
  *
  * Copyright (c) 2001,2002	Tatsuo Ishii
  *
@@ -29,7 +29,6 @@
 #include "access/heapam.h"
 #include "access/transam.h"
 #include "catalog/namespace.h"
-#include "miscadmin.h"
 #include "utils/builtins.h"
 
 
@@ -112,48 +111,33 @@ pgstattuple_real(Relation rel)
 	uint64		free_space = 0; /* free/reusable space in bytes */
 	double		free_percent;	/* free/reusable space in % */
 	TupleDesc	tupdesc;
-	TupleTableSlot *slot;
 	AttInMetadata *attinmeta;
 	char	  **values;
 	int			i;
 	Datum		result;
 
-	/*
-	 * Reject attempts to read non-local temporary relations; we would
-	 * be likely to get wrong data since we have no visibility into the
-	 * owning session's local buffers.
-	 */
-	if (isOtherTempNamespace(RelationGetNamespace(rel)))
-		ereport(ERROR,
-				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				 errmsg("cannot access temporary tables of other sessions")));
-
 	/*
 	 * Build a tuple description for a pgstattupe_type tuple
 	 */
 	tupdesc = RelationNameGetTupleDesc(DUMMY_TUPLE);
 
-	/* allocate a slot for a tuple with this tupdesc */
-	slot = TupleDescGetSlot(tupdesc);
-
 	/*
 	 * Generate attribute metadata needed later to produce tuples from raw
 	 * C strings
 	 */
 	attinmeta = TupleDescGetAttInMetadata(tupdesc);
 
-	nblocks = RelationGetNumberOfBlocks(rel);
 	scan = heap_beginscan(rel, SnapshotAny, 0, NULL);
 
+	nblocks = scan->rs_nblocks; /* # blocks to be scanned */
+
 	/* scan the relation */
 	while ((tuple = heap_getnext(scan, ForwardScanDirection)) != NULL)
 	{
-		uint16		sv_infomask;
+		/* must hold a buffer lock to call HeapTupleSatisfiesNow */
+		LockBuffer(scan->rs_cbuf, BUFFER_LOCK_SHARE);
 
-		CHECK_FOR_INTERRUPTS();
-
-		sv_infomask = tuple->t_data->t_infomask;
-		if (HeapTupleSatisfiesNow(tuple->t_data))
+		if (HeapTupleSatisfiesNow(tuple->t_data, scan->rs_cbuf))
 		{
 			tuple_len += tuple->t_len;
 			tuple_count++;
@@ -163,8 +147,8 @@ pgstattuple_real(Relation rel)
 			dead_tuple_len += tuple->t_len;
 			dead_tuple_count++;
 		}
-		if (sv_infomask != tuple->t_data->t_infomask)
-			SetBufferCommitInfoNeedsSave(scan->rs_cbuf);
+
+		LockBuffer(scan->rs_cbuf, BUFFER_LOCK_UNLOCK);
 
 		/*
 		 * To avoid physically reading the table twice, try to do the
@@ -176,10 +160,10 @@ pgstattuple_real(Relation rel)
 
 		while (block <= tupblock)
 		{
-			CHECK_FOR_INTERRUPTS();
-
 			buffer = ReadBuffer(rel, block);
+			LockBuffer(buffer, BUFFER_LOCK_SHARE);
 			free_space += PageGetFreeSpace((Page) BufferGetPage(buffer));
+			LockBuffer(buffer, BUFFER_LOCK_UNLOCK);
 			ReleaseBuffer(buffer);
 			block++;
 		}
@@ -188,8 +172,6 @@ pgstattuple_real(Relation rel)
 
 	while (block < nblocks)
 	{
-		CHECK_FOR_INTERRUPTS();
-
 		buffer = ReadBuffer(rel, block);
 		free_space += PageGetFreeSpace((Page) BufferGetPage(buffer));
 		ReleaseBuffer(buffer);
@@ -214,9 +196,9 @@ pgstattuple_real(Relation rel)
 	}
 
 	/*
-	 * Prepare a values array for storage in our slot. This should be an
-	 * array of C strings which will be processed later by the appropriate
-	 * "in" functions.
+	 * Prepare a values array for constructing the tuple. This should be
+	 * an array of C strings which will be processed later by the
+	 * appropriate "in" functions.
 	 */
 	values = (char **) palloc(NCOLUMNS * sizeof(char *));
 	for (i = 0; i < NCOLUMNS; i++)
@@ -236,7 +218,7 @@ pgstattuple_real(Relation rel)
 	tuple = BuildTupleFromCStrings(attinmeta, values);
 
 	/* make the tuple into a datum */
-	result = TupleGetDatum(slot, tuple);
+	result = HeapTupleGetDatum(tuple);
 
 	/* Clean up */
 	for (i = 0; i < NCOLUMNS; i++)
