@@ -4,12 +4,12 @@
  *		Routines for handling specialized SET variables.
  *
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/commands/variable.c,v 1.88.2.5 2009/09/03 22:09:05 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/commands/variable.c,v 1.105.4.3 2006/02/12 22:33:14 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -17,29 +17,19 @@
 #include "postgres.h"
 
 #include <ctype.h>
-#include <time.h>
 
 #include "access/xact.h"
 #include "catalog/pg_shadow.h"
 #include "commands/variable.h"
 #include "miscadmin.h"
+#include "parser/scansup.h"
+#include "pgtime.h"
 #include "utils/builtins.h"
 #include "utils/guc.h"
 #include "utils/syscache.h"
 #include "utils/tqual.h"
 #include "mb/pg_wchar.h"
 
-/*
- * Some systems have tzname[] but don't declare it in <time.h>.  Use this
- * to duplicate the test in AC_STRUCT_TIMEZONE.
- */
-#ifdef HAVE_TZNAME
-#ifndef tzname					/* For SGI.  */
-extern char *tzname[];
-#endif
-#endif
-
-
 /*
  * DATESTYLE
  */
@@ -48,7 +38,7 @@ extern char *tzname[];
  * assign_datestyle: GUC assign_hook for datestyle
  */
 const char *
-assign_datestyle(const char *value, bool doit, bool interactive)
+assign_datestyle(const char *value, bool doit, GucSource source)
 {
 	int			newDateStyle = DateStyle;
 	int			newDateOrder = DateOrder;
@@ -58,7 +48,7 @@ assign_datestyle(const char *value, bool doit, bool interactive)
 	char	   *rawstring;
 	char	   *result;
 	List	   *elemlist;
-	List	   *l;
+	ListCell   *l;
 
 	/* Need a modifiable copy of string */
 	rawstring = pstrdup(value);
@@ -68,11 +58,11 @@ assign_datestyle(const char *value, bool doit, bool interactive)
 	{
 		/* syntax error in list */
 		pfree(rawstring);
-		freeList(elemlist);
-		if (interactive)
+		list_free(elemlist);
+		if (source >= PGC_S_INTERACTIVE)
 			ereport(ERROR,
 					(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-					 errmsg("invalid list syntax for parameter \"datestyle\"")));
+			 errmsg("invalid list syntax for parameter \"datestyle\"")));
 		return NULL;
 	}
 
@@ -82,22 +72,22 @@ assign_datestyle(const char *value, bool doit, bool interactive)
 
 		/* Ugh. Somebody ought to write a table driven version -- mjl */
 
-		if (strcasecmp(tok, "ISO") == 0)
+		if (pg_strcasecmp(tok, "ISO") == 0)
 		{
 			newDateStyle = USE_ISO_DATES;
 			scnt++;
 		}
-		else if (strcasecmp(tok, "SQL") == 0)
+		else if (pg_strcasecmp(tok, "SQL") == 0)
 		{
 			newDateStyle = USE_SQL_DATES;
 			scnt++;
 		}
-		else if (strncasecmp(tok, "POSTGRES", 8) == 0)
+		else if (pg_strncasecmp(tok, "POSTGRES", 8) == 0)
 		{
 			newDateStyle = USE_POSTGRES_DATES;
 			scnt++;
 		}
-		else if (strcasecmp(tok, "GERMAN") == 0)
+		else if (pg_strcasecmp(tok, "GERMAN") == 0)
 		{
 			newDateStyle = USE_GERMAN_DATES;
 			scnt++;
@@ -105,25 +95,25 @@ assign_datestyle(const char *value, bool doit, bool interactive)
 			if (ocnt == 0)
 				newDateOrder = DATEORDER_DMY;
 		}
-		else if (strcasecmp(tok, "YMD") == 0)
+		else if (pg_strcasecmp(tok, "YMD") == 0)
 		{
 			newDateOrder = DATEORDER_YMD;
 			ocnt++;
 		}
-		else if (strcasecmp(tok, "DMY") == 0 ||
-				 strncasecmp(tok, "EURO", 4) == 0)
+		else if (pg_strcasecmp(tok, "DMY") == 0 ||
+				 pg_strncasecmp(tok, "EURO", 4) == 0)
 		{
 			newDateOrder = DATEORDER_DMY;
 			ocnt++;
 		}
-		else if (strcasecmp(tok, "MDY") == 0 ||
-				 strcasecmp(tok, "US") == 0 ||
-				 strncasecmp(tok, "NONEURO", 7) == 0)
+		else if (pg_strcasecmp(tok, "MDY") == 0 ||
+				 pg_strcasecmp(tok, "US") == 0 ||
+				 pg_strncasecmp(tok, "NONEURO", 7) == 0)
 		{
 			newDateOrder = DATEORDER_MDY;
 			ocnt++;
 		}
-		else if (strcasecmp(tok, "DEFAULT") == 0)
+		else if (pg_strcasecmp(tok, "DEFAULT") == 0)
 		{
 			/*
 			 * Easiest way to get the current DEFAULT state is to fetch
@@ -137,7 +127,7 @@ assign_datestyle(const char *value, bool doit, bool interactive)
 			const char *subval;
 
 			subval = assign_datestyle(GetConfigOptionResetString("datestyle"),
-									  true, interactive);
+									  true, source);
 			if (scnt == 0)
 				newDateStyle = DateStyle;
 			if (ocnt == 0)
@@ -155,11 +145,11 @@ assign_datestyle(const char *value, bool doit, bool interactive)
 		}
 		else
 		{
-			if (interactive)
+			if (source >= PGC_S_INTERACTIVE)
 				ereport(ERROR,
 						(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-						 errmsg("unrecognized \"datestyle\" key word: \"%s\"",
-								tok)));
+					errmsg("unrecognized \"datestyle\" key word: \"%s\"",
+						   tok)));
 			ok = false;
 			break;
 		}
@@ -169,11 +159,11 @@ assign_datestyle(const char *value, bool doit, bool interactive)
 		ok = false;
 
 	pfree(rawstring);
-	freeList(elemlist);
+	list_free(elemlist);
 
 	if (!ok)
 	{
-		if (interactive)
+		if (source >= PGC_S_INTERACTIVE)
 			ereport(ERROR,
 					(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 					 errmsg("conflicting \"datestyle\" specifications")));
@@ -236,181 +226,20 @@ assign_datestyle(const char *value, bool doit, bool interactive)
  * TIMEZONE
  */
 
-/*
- * Storage for TZ env var is allocated with an arbitrary size of 64 bytes.
- */
-#define TZBUF_LEN	64
-
-static char tzbuf[TZBUF_LEN];
-
-/*
- * First time through, we remember the original environment TZ value, if any.
- */
-static bool have_saved_tz = false;
-static char orig_tzbuf[TZBUF_LEN];
-
-/*
- * Convenience subroutine for assigning the value of TZ
- */
-static void
-set_tz(const char *tz)
-{
-	strcpy(tzbuf, "TZ=");
-	strncpy(tzbuf + 3, tz, sizeof(tzbuf) - 4);
-	if (putenv(tzbuf) != 0)		/* shouldn't happen? */
-		elog(LOG, "could not set TZ environment variable");
-	tzset();
-}
-
-/*
- * Remove any value of TZ we have established
- *
- * Note: this leaves us with *no* value of TZ in the environment, and
- * is therefore only appropriate for reverting to that state, not for
- * reverting to a state where TZ was set to something else.
- */
-static void
-clear_tz(void)
-{
-	/*
-	 * unsetenv() works fine, but is BSD, not POSIX, and is not available
-	 * under Solaris, among others. Apparently putenv() called as below
-	 * clears the process-specific environment variables.  Other
-	 * reasonable arguments to putenv() (e.g. "TZ=", "TZ", "") result in a
-	 * core dump (under Linux anyway). - thomas 1998-01-26
-	 */
-	if (tzbuf[0] == 'T')
-	{
-		strcpy(tzbuf, "=");
-		if (putenv(tzbuf) != 0)
-			elog(LOG, "could not clear TZ environment variable");
-		tzset();
-	}
-}
-
-/*
- * Check whether tzset() succeeded
- *
- * Unfortunately, tzset doesn't offer any well-defined way to detect that the
- * value of TZ was bad.  Often it will just select UTC (GMT) as the effective
- * timezone.  We use the following heuristics:
- *
- * If tzname[1] is a nonempty string, *or* the global timezone variable is
- * not zero, then tzset must have recognized the TZ value as something
- * different from UTC.	Return true.
- *
- * Otherwise, check to see if the TZ name is a known spelling of "UTC"
- * (ie, appears in our internal tables as a timezone equivalent to UTC).
- * If so, accept it.
- *
- * This will reject nonstandard spellings of UTC unless tzset() chose to
- * set tzname[1] as well as tzname[0].	The glibc version of tzset() will
- * do so, but on other systems we may be tightening the spec a little.
- *
- * Another problem is that on some platforms (eg HPUX), if tzset thinks the
- * input is bogus then it will adopt the system default timezone, which we
- * really can't tell is not the intended translation of the input.
- *
- * Still, it beats failing to detect bad TZ names at all, and a silent
- * failure mode of adopting the system-wide default is much better than
- * a silent failure mode of adopting UTC.
- *
- * NB: this must NOT ereport(ERROR).  The caller must get control back so that
- * it can restore the old value of TZ if we don't like the new one.
- */
-static bool
-tzset_succeeded(const char *tz)
-{
-	char		tztmp[TZBUF_LEN];
-	char	   *cp;
-	int			tzval;
-
-	/*
-	 * Check first set of heuristics to say that tzset definitely worked.
-	 */
-#ifdef HAVE_TZNAME
-	if (tzname[1] && tzname[1][0] != '\0')
-		return true;
-#endif
-	if (TIMEZONE_GLOBAL != 0)
-		return true;
-
-	/*
-	 * Check for known spellings of "UTC".	Note we must downcase the
-	 * input before passing it to DecodePosixTimezone().
-	 */
-	StrNCpy(tztmp, tz, sizeof(tztmp));
-	for (cp = tztmp; *cp; cp++)
-		*cp = tolower((unsigned char) *cp);
-	if (DecodePosixTimezone(tztmp, &tzval) == 0)
-		if (tzval == 0)
-			return true;
-
-	return false;
-}
-
-/*
- * Check whether timezone is acceptable.
- *
- * What we are doing here is checking for leap-second-aware timekeeping.
- * We need to reject such TZ settings because they'll wreak havoc with our
- * date/time arithmetic.
- *
- * NB: this must NOT ereport(ERROR).  The caller must get control back so that
- * it can restore the old value of TZ if we don't like the new one.
- */
-static bool
-tz_acceptable(void)
-{
-	struct tm	tt;
-	time_t		time2000;
-
-	/*
-	 * To detect leap-second timekeeping, compute the time_t value for
-	 * local midnight, 2000-01-01.	Insist that this be a multiple of 60;
-	 * any partial-minute offset has to be due to leap seconds.
-	 */
-	MemSet(&tt, 0, sizeof(tt));
-	tt.tm_year = 100;
-	tt.tm_mon = 0;
-	tt.tm_mday = 1;
-	tt.tm_isdst = -1;
-	time2000 = mktime(&tt);
-	if ((time2000 % 60) != 0)
-		return false;
-
-	return true;
-}
-
 /*
  * assign_timezone: GUC assign_hook for timezone
  */
 const char *
-assign_timezone(const char *value, bool doit, bool interactive)
+assign_timezone(const char *value, bool doit, GucSource source)
 {
 	char	   *result;
 	char	   *endptr;
 	double		hours;
 
-	/*
-	 * On first call, see if there is a TZ in the original environment.
-	 * Save that value permanently.
-	 */
-	if (!have_saved_tz)
-	{
-		char	   *orig_tz = getenv("TZ");
-
-		if (orig_tz)
-			StrNCpy(orig_tzbuf, orig_tz, sizeof(orig_tzbuf));
-		else
-			orig_tzbuf[0] = '\0';
-		have_saved_tz = true;
-	}
-
 	/*
 	 * Check for INTERVAL 'foo'
 	 */
-	if (strncasecmp(value, "interval", 8) == 0)
+	if (pg_strncasecmp(value, "interval", 8) == 0)
 	{
 		const char *valueptr = value;
 		char	   *val;
@@ -441,10 +270,11 @@ assign_timezone(const char *value, bool doit, bool interactive)
 													CStringGetDatum(val),
 											ObjectIdGetDatum(InvalidOid),
 													 Int32GetDatum(-1)));
+
 		pfree(val);
 		if (interval->month != 0)
 		{
-			if (interactive)
+			if (source >= PGC_S_INTERACTIVE)
 				ereport(ERROR,
 						(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 						 errmsg("invalid interval value for time zone: month not allowed")));
@@ -459,6 +289,7 @@ assign_timezone(const char *value, bool doit, bool interactive)
 #else
 			CTimeZone = -interval->time;
 #endif
+
 			HasCTZSet = true;
 		}
 		pfree(interval);
@@ -478,39 +309,28 @@ assign_timezone(const char *value, bool doit, bool interactive)
 				HasCTZSet = true;
 			}
 		}
-		else if (strcasecmp(value, "UNKNOWN") == 0)
+		else if (pg_strcasecmp(value, "UNKNOWN") == 0)
 		{
 			/*
 			 * UNKNOWN is the value shown as the "default" for TimeZone in
-			 * guc.c.  We interpret it as meaning the original TZ
-			 * inherited from the environment.	Note that if there is an
-			 * original TZ setting, we will return that rather than
+			 * guc.c.  We interpret it as being a complete no-op; we don't
+			 * change the timezone setting.  Note that if there is a known
+			 * timezone setting, we will return that name rather than
 			 * UNKNOWN as the canonical spelling.
+			 *
+			 * During GUC initialization, since the timezone library isn't
+			 * set up yet, pg_get_current_timezone will return NULL and we
+			 * will leave the setting as UNKNOWN.  If this isn't
+			 * overridden from the config file then
+			 * pg_timezone_initialize() will eventually select a default
+			 * value from the environment.
 			 */
 			if (doit)
 			{
-				bool		ok;
+				const char *curzone = pg_get_current_timezone();
 
-				/* Revert to original setting of TZ, whatever it was */
-				if (orig_tzbuf[0])
-				{
-					set_tz(orig_tzbuf);
-					ok = tzset_succeeded(orig_tzbuf) && tz_acceptable();
-				}
-				else
-				{
-					clear_tz();
-					ok = tz_acceptable();
-				}
-
-				if (ok)
-					HasCTZSet = false;
-				else
-				{
-					/* Bogus, so force UTC (equivalent to INTERVAL 0) */
-					CTimeZone = 0;
-					HasCTZSet = true;
-				}
+				if (curzone)
+					value = curzone;
 			}
 		}
 		else
@@ -520,22 +340,27 @@ assign_timezone(const char *value, bool doit, bool interactive)
 			 *
 			 * We have to actually apply the change before we can have any
 			 * hope of checking it.  So, save the old value in case we
-			 * have to back out.  Note that it's possible the old setting
-			 * is in tzbuf, so we'd better copy it.
+			 * have to back out.  We have to copy since
+			 * pg_get_current_timezone returns a pointer to its static
+			 * state.
+			 *
+			 * This would all get a lot simpler if the TZ library had a
+			 * better API that would let us look up and test a timezone
+			 * name without making it the default.
 			 */
-			char		save_tzbuf[TZBUF_LEN];
+			const char *cur_tz;
 			char	   *save_tz;
 			bool		known,
 						acceptable;
 
-			save_tz = getenv("TZ");
-			if (save_tz)
-				StrNCpy(save_tzbuf, save_tz, sizeof(save_tzbuf));
+			cur_tz = pg_get_current_timezone();
+			if (cur_tz)
+				save_tz = pstrdup(cur_tz);
+			else
+				save_tz = NULL;
 
-			set_tz(value);
-
-			known = tzset_succeeded(value);
-			acceptable = tz_acceptable();
+			known = pg_tzset(value);
+			acceptable = known ? tz_acceptable() : false;
 
 			if (doit && known && acceptable)
 			{
@@ -550,13 +375,37 @@ assign_timezone(const char *value, bool doit, bool interactive)
 				 * a fixed offset, we still are.
 				 */
 				if (save_tz)
-					set_tz(save_tzbuf);
+					pg_tzset(save_tz);
 				else
-					clear_tz();
+				{
+					/*
+					 * TZ library wasn't initialized yet.  Annoyingly, we
+					 * will come here during startup because guc-file.l
+					 * checks the value with doit = false before actually
+					 * applying. The best approach seems to be as follows:
+					 *
+					 * 1. known && acceptable: leave the setting in place,
+					 * since we'll apply it soon anyway.  This is mainly
+					 * so that any log messages printed during this
+					 * interval are timestamped with the user's requested
+					 * timezone.
+					 *
+					 * 2. known && !acceptable: revert to GMT for lack of any
+					 * better idea.  (select_default_timezone() may get
+					 * called later to undo this.)
+					 *
+					 * 3. !known: no need to do anything since TZ library did
+					 * not change its state.
+					 *
+					 * Again, this should all go away sometime soon.
+					 */
+					if (known && !acceptable)
+						pg_tzset("GMT");
+				}
 				/* Complain if it was bad */
 				if (!known)
 				{
-					ereport(interactive ? ERROR : LOG,
+					ereport((source >= PGC_S_INTERACTIVE) ? ERROR : LOG,
 							(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 							 errmsg("unrecognized time zone name: \"%s\"",
 									value)));
@@ -564,7 +413,7 @@ assign_timezone(const char *value, bool doit, bool interactive)
 				}
 				if (!acceptable)
 				{
-					ereport(interactive ? ERROR : LOG,
+					ereport((source >= PGC_S_INTERACTIVE) ? ERROR : LOG,
 							(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 					errmsg("time zone \"%s\" appears to use leap seconds",
 						   value),
@@ -584,17 +433,16 @@ assign_timezone(const char *value, bool doit, bool interactive)
 	/*
 	 * Prepare the canonical string to return.	GUC wants it malloc'd.
 	 */
-	result = (char *) malloc(sizeof(tzbuf));
-	if (!result)
-		return NULL;
-
 	if (HasCTZSet)
-		snprintf(result, sizeof(tzbuf), "%.5f",
+	{
+		result = (char *) malloc(64);
+		if (!result)
+			return NULL;
+		snprintf(result, 64, "%.5f",
 				 (double) (-CTimeZone) / 3600.0);
-	else if (tzbuf[0] == 'T')
-		strcpy(result, tzbuf + 3);
+	}
 	else
-		strcpy(result, "UNKNOWN");
+		result = strdup(value);
 
 	return result;
 }
@@ -605,11 +453,11 @@ assign_timezone(const char *value, bool doit, bool interactive)
 const char *
 show_timezone(void)
 {
-	char	   *tzn;
+	const char *tzn;
 
 	if (HasCTZSet)
 	{
-		Interval	interval;
+		Interval interval;
 
 		interval.month = 0;
 #ifdef HAVE_INT64_TIMESTAMP
@@ -622,7 +470,7 @@ show_timezone(void)
 										  IntervalPGetDatum(&interval)));
 	}
 	else
-		tzn = getenv("TZ");
+		tzn = pg_get_current_timezone();
 
 	if (tzn != NULL)
 		return tzn;
@@ -636,23 +484,49 @@ show_timezone(void)
  */
 
 const char *
-assign_XactIsoLevel(const char *value, bool doit, bool interactive)
+assign_XactIsoLevel(const char *value, bool doit, GucSource source)
 {
-	if (doit && interactive && SerializableSnapshot != NULL)
-		ereport(ERROR,
-				(errcode(ERRCODE_ACTIVE_SQL_TRANSACTION),
-				 errmsg("SET TRANSACTION ISOLATION LEVEL must be called before any query")));
+	if (SerializableSnapshot != NULL)
+	{
+		if (source >= PGC_S_INTERACTIVE)
+			ereport(ERROR,
+					(errcode(ERRCODE_ACTIVE_SQL_TRANSACTION),
+					 errmsg("SET TRANSACTION ISOLATION LEVEL must be called before any query")));
+		/* source == PGC_S_OVERRIDE means do it anyway, eg at xact abort */
+		else if (source != PGC_S_OVERRIDE)
+			return NULL;
+	}
+	if (IsSubTransaction())
+	{
+		if (source >= PGC_S_INTERACTIVE)
+			ereport(ERROR,
+					(errcode(ERRCODE_ACTIVE_SQL_TRANSACTION),
+					 errmsg("SET TRANSACTION ISOLATION LEVEL must not be called in a subtransaction")));
+		/* source == PGC_S_OVERRIDE means do it anyway, eg at xact abort */
+		else if (source != PGC_S_OVERRIDE)
+			return NULL;
+	}
 
 	if (strcmp(value, "serializable") == 0)
 	{
 		if (doit)
 			XactIsoLevel = XACT_SERIALIZABLE;
 	}
+	else if (strcmp(value, "repeatable read") == 0)
+	{
+		if (doit)
+			XactIsoLevel = XACT_REPEATABLE_READ;
+	}
 	else if (strcmp(value, "read committed") == 0)
 	{
 		if (doit)
 			XactIsoLevel = XACT_READ_COMMITTED;
 	}
+	else if (strcmp(value, "read uncommitted") == 0)
+	{
+		if (doit)
+			XactIsoLevel = XACT_READ_UNCOMMITTED;
+	}
 	else if (strcmp(value, "default") == 0)
 	{
 		if (doit)
@@ -667,10 +541,19 @@ assign_XactIsoLevel(const char *value, bool doit, bool interactive)
 const char *
 show_XactIsoLevel(void)
 {
-	if (XactIsoLevel == XACT_SERIALIZABLE)
-		return "serializable";
-	else
-		return "read committed";
+	switch (XactIsoLevel)
+	{
+		case XACT_READ_UNCOMMITTED:
+			return "read uncommitted";
+		case XACT_READ_COMMITTED:
+			return "read committed";
+		case XACT_REPEATABLE_READ:
+			return "repeatable read";
+		case XACT_SERIALIZABLE:
+			return "serializable";
+		default:
+			return "bogus";
+	}
 }
 
 
@@ -679,10 +562,10 @@ show_XactIsoLevel(void)
  */
 
 bool
-assign_random_seed(double value, bool doit, bool interactive)
+assign_random_seed(double value, bool doit, GucSource source)
 {
 	/* Can't really roll back on error, so ignore non-interactive setting */
-	if (doit && interactive)
+	if (doit && source >= PGC_S_INTERACTIVE)
 		DirectFunctionCall1(setseed, Float8GetDatum(value));
 	return true;
 }
@@ -699,7 +582,7 @@ show_random_seed(void)
  */
 
 const char *
-assign_client_encoding(const char *value, bool doit, bool interactive)
+assign_client_encoding(const char *value, bool doit, GucSource source)
 {
 	int			encoding;
 
@@ -715,7 +598,7 @@ assign_client_encoding(const char *value, bool doit, bool interactive)
 	 */
 	if (SetClientEncoding(encoding, doit) < 0)
 	{
-		if (interactive)
+		if (source >= PGC_S_INTERACTIVE)
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 				  errmsg("conversion between %s and %s is not supported",
@@ -740,10 +623,10 @@ assign_client_encoding(const char *value, bool doit, bool interactive)
  * properly truncated any incoming value, but not to truncate already-stored
  * values.  See GUC_IS_NAME processing.)
  */
-extern char *session_authorization_string; /* in guc.c */
+extern char *session_authorization_string;		/* in guc.c */
 
 const char *
-assign_session_authorization(const char *value, bool doit, bool interactive)
+assign_session_authorization(const char *value, bool doit, GucSource source)
 {
 	AclId		usesysid = 0;
 	bool		is_superuser = false;
@@ -788,7 +671,7 @@ assign_session_authorization(const char *value, bool doit, bool interactive)
 								 0, 0, 0);
 		if (!HeapTupleIsValid(userTup))
 		{
-			if (interactive)
+			if (source >= PGC_S_INTERACTIVE)
 				ereport(ERROR,
 						(errcode(ERRCODE_UNDEFINED_OBJECT),
 						 errmsg("user \"%s\" does not exist", value)));
@@ -830,10 +713,6 @@ show_session_authorization(void)
 	AclId		savedsysid;
 	char	   *endptr;
 
-	/* If session_authorization hasn't been set in this process, return "" */
-	if (value == NULL || value[0] == '\0')
-		return "";
-
 	Assert(strspn(value, "x") == NAMEDATALEN &&
 		   (value[NAMEDATALEN] == 'T' || value[NAMEDATALEN] == 'F'));
 
