@@ -4,11 +4,11 @@
  *	  POSTGRES free space map for quickly finding free space in relations
  *
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/storage/freespace/freespace.c,v 1.24 2003/10/29 17:36:57 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/storage/freespace/freespace.c,v 1.37 2004/12/31 22:00:54 pgsql Exp $
  *
  *
  * NOTES:
@@ -180,7 +180,6 @@ typedef struct FSMRelation FSMRelation;
 /* Header for whole map */
 struct FSMHeader
 {
-	HTAB	   *relHash;		/* hashtable of FSMRelation entries */
 	FSMRelation *usageList;		/* FSMRelations in usage-recency order */
 	FSMRelation *usageListTail; /* tail of usage-recency list */
 	FSMRelation *firstRel;		/* FSMRelations in arena storage order */
@@ -218,6 +217,8 @@ int			MaxFSMRelations;	/* these are set by guc.c */
 int			MaxFSMPages;
 
 static FSMHeader *FreeSpaceMap; /* points to FSMHeader in shared memory */
+static HTAB *FreeSpaceMapRelHash;		/* points to (what used to be)
+										 * FSMHeader->relHash */
 
 
 static FSMRelation *lookup_fsm_rel(RelFileNode *rel);
@@ -265,31 +266,37 @@ InitFreeSpaceMap(void)
 {
 	HASHCTL		info;
 	int			nchunks;
+	bool		found;
 
 	/* Create table header */
-	FreeSpaceMap = (FSMHeader *) ShmemAlloc(sizeof(FSMHeader));
+	FreeSpaceMap = (FSMHeader *) ShmemInitStruct("Free Space Map Header", sizeof(FSMHeader), &found);
 	if (FreeSpaceMap == NULL)
 		ereport(FATAL,
 				(errcode(ERRCODE_OUT_OF_MEMORY),
 			   errmsg("insufficient shared memory for free space map")));
-	MemSet(FreeSpaceMap, 0, sizeof(FSMHeader));
+	if (!found)
+		MemSet(FreeSpaceMap, 0, sizeof(FSMHeader));
 
 	/* Create hashtable for FSMRelations */
 	info.keysize = sizeof(RelFileNode);
 	info.entrysize = sizeof(FSMRelation);
 	info.hash = tag_hash;
 
-	FreeSpaceMap->relHash = ShmemInitHash("Free Space Map Hash",
-										  MaxFSMRelations / 10,
-										  MaxFSMRelations,
-										  &info,
-										  (HASH_ELEM | HASH_FUNCTION));
+	FreeSpaceMapRelHash = ShmemInitHash("Free Space Map Hash",
+										MaxFSMRelations + 1,
+										MaxFSMRelations + 1,
+										&info,
+										(HASH_ELEM | HASH_FUNCTION));
 
-	if (!FreeSpaceMap->relHash)
+	if (!FreeSpaceMapRelHash)
 		ereport(FATAL,
 				(errcode(ERRCODE_OUT_OF_MEMORY),
 			   errmsg("insufficient shared memory for free space map")));
 
+	if (found)
+		return;
+
+
 	/* Allocate page-storage arena */
 	nchunks = (MaxFSMPages - 1) / CHUNKPAGES + 1;
 	/* This check ensures spareChunks will be greater than zero */
@@ -323,7 +330,7 @@ FreeSpaceShmemSize(void)
 	size = MAXALIGN(sizeof(FSMHeader));
 
 	/* hash table, including the FSMRelation objects */
-	size += hash_estimate_size(MaxFSMRelations, sizeof(FSMRelation));
+	size += hash_estimate_size(MaxFSMRelations + 1, sizeof(FSMRelation));
 
 	/* page-storage arena */
 	nchunks = (MaxFSMPages - 1) / CHUNKPAGES + 1;
@@ -652,9 +659,6 @@ FreeSpaceMapForgetRel(RelFileNode *rel)
  *
  * This is called during DROP DATABASE.  As above, might as well reclaim
  * map space sooner instead of later.
- *
- * XXX when we implement tablespaces, target Oid will need to be tablespace
- * ID not database ID.
  */
 void
 FreeSpaceMapForgetDatabase(Oid dbid)
@@ -666,7 +670,7 @@ FreeSpaceMapForgetDatabase(Oid dbid)
 	for (fsmrel = FreeSpaceMap->usageList; fsmrel; fsmrel = nextrel)
 	{
 		nextrel = fsmrel->nextUsage;	/* in case we delete it */
-		if (fsmrel->key.tblNode == dbid)
+		if (fsmrel->key.dbNode == dbid)
 			delete_fsm_rel(fsmrel);
 	}
 	LWLockRelease(FreeSpaceLock);
@@ -717,7 +721,7 @@ PrintFreeSpaceMapStatistics(int elevel)
  * forma --- if anyone else is still accessing FSM, there's a problem.
  */
 void
-DumpFreeSpaceMap(void)
+DumpFreeSpaceMap(int code, Datum arg)
 {
 	FILE	   *fp;
 	char		cachefilename[MAXPGPATH];
@@ -787,7 +791,12 @@ DumpFreeSpaceMap(void)
 	/* Clean up */
 	LWLockRelease(FreeSpaceLock);
 
-	FreeFile(fp);
+	if (FreeFile(fp))
+	{
+		elog(LOG, "could not write \"%s\": %m", cachefilename);
+		/* Remove busted cache file */
+		unlink(cachefilename);
+	}
 
 	return;
 
@@ -882,7 +891,7 @@ LoadFreeSpaceMap(void)
 			len = nPages * sizeof(IndexFSMPageData);
 		else
 			len = nPages * sizeof(FSMPageData);
-		data = (char *) palloc(len + 1);		/* +1 to avoid palloc(0) */
+		data = (char *) palloc(len);
 		if (fread(data, 1, len, fp) != len)
 		{
 			elog(LOG, "premature EOF in \"%s\"", cachefilename);
@@ -974,7 +983,7 @@ lookup_fsm_rel(RelFileNode *rel)
 {
 	FSMRelation *fsmrel;
 
-	fsmrel = (FSMRelation *) hash_search(FreeSpaceMap->relHash,
+	fsmrel = (FSMRelation *) hash_search(FreeSpaceMapRelHash,
 										 (void *) rel,
 										 HASH_FIND,
 										 NULL);
@@ -995,7 +1004,7 @@ create_fsm_rel(RelFileNode *rel)
 	FSMRelation *fsmrel;
 	bool		found;
 
-	fsmrel = (FSMRelation *) hash_search(FreeSpaceMap->relHash,
+	fsmrel = (FSMRelation *) hash_search(FreeSpaceMapRelHash,
 										 (void *) rel,
 										 HASH_ENTER,
 										 &found);
@@ -1050,7 +1059,7 @@ delete_fsm_rel(FSMRelation *fsmrel)
 	unlink_fsm_rel_usage(fsmrel);
 	unlink_fsm_rel_storage(fsmrel);
 	FreeSpaceMap->numRels--;
-	result = (FSMRelation *) hash_search(FreeSpaceMap->relHash,
+	result = (FSMRelation *) hash_search(FreeSpaceMapRelHash,
 										 (void *) &(fsmrel->key),
 										 HASH_REMOVE,
 										 NULL);
@@ -1394,6 +1403,7 @@ static void
 compact_fsm_storage(void)
 {
 	int			nextChunkIndex = 0;
+	bool		did_push = false;
 	FSMRelation *fsmrel;
 
 	for (fsmrel = FreeSpaceMap->firstRel;
@@ -1419,31 +1429,37 @@ compact_fsm_storage(void)
 			newAllocPages = newAlloc * INDEXCHUNKPAGES;
 		else
 			newAllocPages = newAlloc * CHUNKPAGES;
-		newChunkIndex = nextChunkIndex;
-		nextChunkIndex += newAlloc;
 
 		/*
 		 * Determine current size, current and new locations
 		 */
 		curChunks = fsm_current_chunks(fsmrel);
 		oldChunkIndex = fsmrel->firstChunk;
-		newLocation = FreeSpaceMap->arena + newChunkIndex * CHUNKBYTES;
 		oldLocation = FreeSpaceMap->arena + oldChunkIndex * CHUNKBYTES;
+		newChunkIndex = nextChunkIndex;
+		newLocation = FreeSpaceMap->arena + newChunkIndex * CHUNKBYTES;
 
 		/*
 		 * It's possible that we have to move data down, not up, if the
-		 * allocations of previous rels expanded.  This normally means that
-		 * our allocation expanded too (or at least got no worse), and
-		 * ditto for later rels.  So there should be room to move all our
-		 * data down without dropping any --- but we might have to push down
-		 * following rels to acquire the room.  We don't want to do the push
-		 * more than once, so pack everything against the end of the arena
-		 * if so.
+		 * allocations of previous rels expanded.  This normally means
+		 * that our allocation expanded too (or at least got no worse),
+		 * and ditto for later rels.  So there should be room to move all
+		 * our data down without dropping any --- but we might have to
+		 * push down following rels to acquire the room.  We don't want to
+		 * do the push more than once, so pack everything against the end
+		 * of the arena if so.
 		 *
-		 * In corner cases where roundoff has affected our allocation, it's
-		 * possible that we have to move down and compress our data too.
-		 * Since this case is extremely infrequent, we do not try to be smart
-		 * about it --- we just drop pages from the end of the rel's data.
+		 * In corner cases where we are on the short end of a roundoff choice
+		 * that we were formerly on the long end of, it's possible that we
+		 * have to move down and compress our data too.  In fact, even
+		 * after pushing down the following rels, there might not be as
+		 * much space as we computed for this rel above --- that would
+		 * imply that some following rel(s) are also on the losing end of
+		 * roundoff choices. We could handle this fairly by doing the
+		 * per-rel compactions out-of-order, but that seems like way too
+		 * much complexity to deal with a very infrequent corner case.
+		 * Instead, we simply drop pages from the end of the current rel's
+		 * data until it fits.
 		 */
 		if (newChunkIndex > oldChunkIndex)
 		{
@@ -1455,21 +1471,46 @@ compact_fsm_storage(void)
 				fsmrel->storedPages = newAllocPages;
 				curChunks = fsm_current_chunks(fsmrel);
 			}
+			/* is there enough space? */
 			if (fsmrel->nextPhysical != NULL)
 				limitChunkIndex = fsmrel->nextPhysical->firstChunk;
 			else
 				limitChunkIndex = FreeSpaceMap->totalChunks;
 			if (newChunkIndex + curChunks > limitChunkIndex)
 			{
-				/* need to push down additional rels */
-				push_fsm_rels_after(fsmrel);
-				/* recheck for safety */
+				/* not enough space, push down following rels */
+				if (!did_push)
+				{
+					push_fsm_rels_after(fsmrel);
+					did_push = true;
+				}
+				/* now is there enough space? */
 				if (fsmrel->nextPhysical != NULL)
 					limitChunkIndex = fsmrel->nextPhysical->firstChunk;
 				else
 					limitChunkIndex = FreeSpaceMap->totalChunks;
 				if (newChunkIndex + curChunks > limitChunkIndex)
-					elog(PANIC, "insufficient room in FSM");
+				{
+					/* uh-oh, forcibly cut the allocation to fit */
+					newAlloc = limitChunkIndex - newChunkIndex;
+
+					/*
+					 * If newAlloc < 0 at this point, we are moving the
+					 * rel's firstChunk into territory currently assigned
+					 * to a later rel.	This is okay so long as we do not
+					 * copy any data. The rels will be back in
+					 * nondecreasing firstChunk order at completion of the
+					 * compaction pass.
+					 */
+					if (newAlloc < 0)
+						newAlloc = 0;
+					if (fsmrel->isIndex)
+						newAllocPages = newAlloc * INDEXCHUNKPAGES;
+					else
+						newAllocPages = newAlloc * CHUNKPAGES;
+					fsmrel->storedPages = newAllocPages;
+					curChunks = fsm_current_chunks(fsmrel);
+				}
 			}
 			memmove(newLocation, oldLocation, curChunks * CHUNKBYTES);
 		}
@@ -1504,6 +1545,7 @@ compact_fsm_storage(void)
 			memmove(newLocation, oldLocation, curChunks * CHUNKBYTES);
 		}
 		fsmrel->firstChunk = newChunkIndex;
+		nextChunkIndex += newAlloc;
 	}
 	Assert(nextChunkIndex <= FreeSpaceMap->totalChunks);
 	FreeSpaceMap->usedChunks = nextChunkIndex;
@@ -1544,8 +1586,8 @@ push_fsm_rels_after(FSMRelation *afterRel)
 		oldChunkIndex = fsmrel->firstChunk;
 		if (newChunkIndex < oldChunkIndex)
 		{
-			/* trouble... */
-			elog(PANIC, "insufficient room in FSM");
+			/* we're pushing down, how can it move up? */
+			elog(PANIC, "inconsistent entry sizes in FSM");
 		}
 		else if (newChunkIndex > oldChunkIndex)
 		{
@@ -1758,14 +1800,14 @@ fsm_current_chunks(FSMRelation *fsmrel)
 {
 	int			chunkCount;
 
+	/* Make sure storedPages==0 produces right answer */
+	if (fsmrel->storedPages <= 0)
+		return 0;
 	/* Convert page count to chunk count */
 	if (fsmrel->isIndex)
 		chunkCount = (fsmrel->storedPages - 1) / INDEXCHUNKPAGES + 1;
 	else
 		chunkCount = (fsmrel->storedPages - 1) / CHUNKPAGES + 1;
-	/* Make sure storedPages==0 produces right answer */
-	if (chunkCount < 0)
-		chunkCount = 0;
 	return chunkCount;
 }
 
@@ -1806,8 +1848,9 @@ DumpFreeSpace(void)
 	for (fsmrel = FreeSpaceMap->usageList; fsmrel; fsmrel = fsmrel->nextUsage)
 	{
 		relNum++;
-		fprintf(stderr, "Map %d: rel %u/%u isIndex %d avgRequest %u lastPageCount %d nextPage %d\nMap= ",
-				relNum, fsmrel->key.tblNode, fsmrel->key.relNode,
+		fprintf(stderr, "Map %d: rel %u/%u/%u isIndex %d avgRequest %u lastPageCount %d nextPage %d\nMap= ",
+				relNum,
+			fsmrel->key.spcNode, fsmrel->key.dbNode, fsmrel->key.relNode,
 				(int) fsmrel->isIndex, fsmrel->avgRequest,
 				fsmrel->lastPageCount, fsmrel->nextPage);
 		if (fsmrel->isIndex)
