@@ -4,10 +4,10 @@
  *	  header file for read.c and readfuncs.c. These functions are internal
  *	  to the stringToNode interface and should not be used by anyone else.
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: readfuncs.h,v 1.14 2001/11/05 17:46:34 momjian Exp $
+ * $PostgreSQL: pgsql/src/include/nodes/readfuncs.h,v 1.21 2004/12/31 22:03:34 pgsql Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -21,11 +21,11 @@
  */
 extern char *pg_strtok(int *length);
 extern char *debackslash(char *token, int length);
-extern void *nodeRead(bool read_car_only);
+extern void *nodeRead(char *token, int tok_len);
 
 /*
  * prototypes for functions in readfuncs.c
  */
-extern Node *parsePlanString(void);
+extern Node *parseNodeString(void);
 
 #endif   /* READFUNCS_H */
