@@ -4,10 +4,10 @@
  * External declarations pertaining to backend/utils/misc/guc.c and
  * backend/utils/misc/guc-file.l
  *
- * Copyright (c) 2000-2003, PostgreSQL Global Development Group
+ * Copyright (c) 2000-2005, PostgreSQL Global Development Group
  * Written by Peter Eisentraut <peter_e@gmx.net>.
  *
- * $Id: guc.h,v 1.41.2.1 2010/03/25 14:46:06 alvherre Exp $
+ * $PostgreSQL: pgsql/src/include/utils/guc.h,v 1.58.4.1 2005/03/25 16:17:39 tgl Exp $
  *--------------------------------------------------------------------
  */
 #ifndef GUC_H
@@ -44,12 +44,7 @@
  * given backend once it's started, but they can vary across backends.
  *
  * SUSET options can be set at postmaster startup, with the SIGHUP
- * mechanism, or from SQL if you're a superuser. These options cannot
- * be set in the connection startup packet, because when it is processed
- * we don't yet know if the user is a superuser.
- *
- * USERLIMIT options can only be manipulated in certain ways by
- * non-superusers.
+ * mechanism, or from SQL if you're a superuser.
  *
  * USERSET options can be set by anyone any time.
  */
@@ -60,7 +55,6 @@ typedef enum
 	PGC_SIGHUP,
 	PGC_BACKEND,
 	PGC_SUSET,
-	PGC_USERLIMIT,
 	PGC_USERSET
 } GucContext;
 
@@ -74,8 +68,15 @@ typedef enum
  * as the current value.  Note that source == PGC_S_OVERRIDE should be
  * used when setting a PGC_INTERNAL option.
  *
- * PGC_S_UNPRIVILEGED isn't actually a source value, but the dividing line
- * between privileged and unprivileged sources for USERLIMIT purposes.
+ * PGC_S_INTERACTIVE isn't actually a source value, but is the
+ * dividing line between "interactive" and "non-interactive" sources for
+ * error reporting purposes.
+ *
+ * PGC_S_TEST is used when testing values to be stored as per-database or
+ * per-user defaults ("doit" will always be false, so this never gets stored
+ * as the actual source of any value).	This is an interactive case, but
+ * it needs its own source value because some assign hooks need to make
+ * different validity checks in this case.
  */
 typedef enum
 {
@@ -83,16 +84,25 @@ typedef enum
 	PGC_S_ENV_VAR,				/* postmaster environment variable */
 	PGC_S_FILE,					/* postgresql.conf */
 	PGC_S_ARGV,					/* postmaster command line */
-	PGC_S_UNPRIVILEGED,			/* dividing line for USERLIMIT */
 	PGC_S_DATABASE,				/* per-database setting */
 	PGC_S_USER,					/* per-user setting */
 	PGC_S_CLIENT,				/* from client connection request */
 	PGC_S_OVERRIDE,				/* special case to forcibly set default */
+	PGC_S_INTERACTIVE,			/* dividing line for error reporting */
+	PGC_S_TEST,					/* test per-database or per-user setting */
 	PGC_S_SESSION				/* SET command */
 } GucSource;
 
+typedef const char *(*GucStringAssignHook) (const char *newval, bool doit, GucSource source);
+typedef bool (*GucBoolAssignHook) (bool newval, bool doit, GucSource source);
+typedef bool (*GucIntAssignHook) (int newval, bool doit, GucSource source);
+typedef bool (*GucRealAssignHook) (double newval, bool doit, GucSource source);
+
+typedef const char *(*GucShowHook) (void);
+
+#define GUC_QUALIFIER_SEPARATOR '.'
+
 /* GUC vars that are actually declared in guc.c, rather than elsewhere */
-extern bool log_statement;
 extern bool log_duration;
 extern bool Debug_print_plan;
 extern bool Debug_print_parse;
@@ -109,27 +119,77 @@ extern bool log_btree_build_stats;
 extern bool SQL_inheritance;
 extern bool Australian_timezones;
 
+extern bool default_with_oids;
+
 extern int	log_min_error_statement;
 extern int	log_min_messages;
 extern int	client_min_messages;
 extern int	log_min_duration_statement;
 
+extern char *ConfigFileName;
+extern char *HbaFileName;
+extern char *IdentFileName;
+extern char *external_pid_file;
+
 
 extern void SetConfigOption(const char *name, const char *value,
 				GucContext context, GucSource source);
+
+extern void DefineCustomBoolVariable(
+						 const char *name,
+						 const char *short_desc,
+						 const char *long_desc,
+						 bool *valueAddr,
+						 GucContext context,
+						 GucBoolAssignHook assign_hook,
+						 GucShowHook show_hook);
+
+extern void DefineCustomIntVariable(
+						const char *name,
+						const char *short_desc,
+						const char *long_desc,
+						int *valueAddr,
+						int minValue,
+						int maxValue,
+						GucContext context,
+						GucIntAssignHook assign_hook,
+						GucShowHook show_hook);
+
+extern void DefineCustomRealVariable(
+						 const char *name,
+						 const char *short_desc,
+						 const char *long_desc,
+						 double *valueAddr,
+						 double minValue,
+						 double maxValue,
+						 GucContext context,
+						 GucRealAssignHook assign_hook,
+						 GucShowHook show_hook);
+
+extern void DefineCustomStringVariable(
+						   const char *name,
+						   const char *short_desc,
+						   const char *long_desc,
+						   char **valueAddr,
+						   GucContext context,
+						   GucStringAssignHook assign_hook,
+						   GucShowHook show_hook);
+
+extern void EmitWarningsOnPlaceholders(const char *className);
+
 extern const char *GetConfigOption(const char *name);
 extern const char *GetConfigOptionResetString(const char *name);
+extern bool IsSuperuserConfigOption(const char *name);
 extern void ProcessConfigFile(GucContext context);
 extern void InitializeGUCOptions(void);
+extern bool SelectConfigFiles(const char *userDoption, const char *progname);
 extern void ResetAllOptions(void);
-extern void AtEOXact_GUC(bool isCommit);
+extern void AtEOXact_GUC(bool isCommit, bool isSubXact);
 extern void BeginReportingGUCOptions(void);
 extern void ParseLongOption(const char *string, char **name, char **value);
 extern bool set_config_option(const char *name, const char *value,
 				  GucContext context, GucSource source,
 				  bool isLocal, bool changeVal);
-extern void ShowGUCConfigOption(const char *name, DestReceiver *dest);
-extern void ShowAllGUCConfig(DestReceiver *dest);
 extern char *GetConfigOptionByName(const char *name, const char **varname);
 extern void GetConfigOptionByNum(int varnum, const char **values, bool *noshow);
 extern int	GetNumConfigOptions(void);
@@ -144,11 +204,35 @@ extern char *flatten_set_variable_args(const char *name, List *args);
 extern void ProcessGUCArray(ArrayType *array, GucSource source);
 extern ArrayType *GUCArrayAdd(ArrayType *array, const char *name, const char *value);
 extern ArrayType *GUCArrayDelete(ArrayType *array, const char *name);
-extern ArrayType *GUCArrayReset(ArrayType *array);
 
 #ifdef EXEC_BACKEND
-void		write_nondefault_variables(GucContext context);
-void		read_nondefault_variables(void);
+extern void write_nondefault_variables(GucContext context);
+extern void read_nondefault_variables(void);
 #endif
 
+/*
+ * The following functions are not in guc.c, but are declared here to avoid
+ * having to include guc.h in some widely used headers that it really doesn't
+ * belong in.
+ */
+
+/* in utils/adt/datetime.c */
+extern bool ClearDateCache(bool newval, bool doit, GucSource source);
+
+/* in commands/tablespace.c */
+extern const char *assign_default_tablespace(const char *newval,
+				   bool doit, GucSource source);
+
+/* in utils/adt/regexp.c */
+extern const char *assign_regex_flavor(const char *value,
+					bool doit, GucSource source);
+
+/* in catalog/namespace.c */
+extern const char *assign_search_path(const char *newval,
+				   bool doit, GucSource source);
+
+/* in access/transam/xlog.c */
+extern const char *assign_xlog_sync_method(const char *method,
+						bool doit, GucSource source);
+
 #endif   /* GUC_H */
